==> af-restful-0.0.2-sources.jar: com/af/v4/system/restful/utils/ResCheckUtil.java <==
package com.af.v4.system.restful.utils;

import com.af.v4.system.expression.Program;
import com.af.v4.system.plugins.core.CommonTools;
import com.af.v4.system.plugins.io.IOTools;
import com.af.v4.system.restful.exception.FileNotFoundException;
import com.af.v4.system.restful.mapper.LogicMapper;
import com.af.v4.system.restful.mapper.ModuleMapper;
import com.af.v4.system.restful.mapper.ResourceMapper;
import com.af.v4.system.restful.mapper.SqlMapper;
import com.af.v4.system.restful.utils.enums.ResourceType;
import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.Element;
import org.dom4j.io.SAXReader;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.atomic.AtomicInteger;

/**
 * 业务资源检测工具
 *
 * @author Mrriver
 */
@Component
public class ResCheckUtil {

    private static final Logger LOGGER = LoggerFactory.getLogger(ResCheckUtil.class);

    private static final List<String> ERROR_ARRAY = new ArrayList<>();

    private static final String[] sentence = {
            "软件开发往往是这样：最开始的90%代码占用了开始的90%的开发时间；剩下10%代码同样需要90%的开发时间。",
            "这不是一个bug，这只是一个未列出来的特性。",
            "作为一个程序员，郁闷的事情是，⾯对一个代码块，却不敢去修改。更糟糕的是，这个代码块还是⾃⼰写的。",
            "在系统测试阶段查找并修复 Bug，花费的时间和工作量是让开发者自己找 Bug 的三倍。在正式上线后查找并修复 Bug，花费的时间和工作量是系统测试阶段的十倍。因此一定要让开发者自己做单元测试。",
            "不要站着调试。站着会让你的耐心减半，而且你需要集中所有精力。",
            "测试可以发现bug的存在，但不能证明bug不存在。",
            "新系统的每个新用户都会发现一类新的Bug。",
            "修复损坏的程序的第一步就是让它不断失败。",
            "有两种方法能写出没有错误的程序；但只有第三种好用。",
            "代码写的越急，程序跑得越慢。",
            "在所有的程序错误中，80%是语法错误，剩下20%里，80%是简单的逻辑错误，再剩下4%里，80%是指针错误，只有余下的0.8%才是困难的问题。"
    };

    private final ResourceMapper resourceMapper;

    private final SqlMapper sqlMapper;

    private final ModuleMapper moduleMapper;

    private final LogicMapper logicMapper;

    public ResCheckUtil(ResourceMapper resourceMapper, SqlMapper sqlMapper, ModuleMapper moduleMapper, LogicMapper logicMapper) {
        this.resourceMapper = resourceMapper;
        this.sqlMapper = sqlMapper;
        this.moduleMapper = moduleMapper;
        this.logicMapper = logicMapper;
    }

    @PostConstruct
    public void checkAll() {
        LOGGER.info("==================开始资源强制检查==================");
        // 获取所有引入的表具模块名
        Map<String, Map<String, String>> map = moduleMapper.getMap();
        int GrammarErrorNums = 0;
        int GrammarErrorSQLNums = 0;
        for (String module : map.keySet()) {
            String modulePath = map.get(module).get("path");
            String logicUrl = (modulePath == null ? module + "/logic.xml" : modulePath + "/"
                    + module + "/logic.xml");
            String sqlUrl = (modulePath == null ? module + "/sql.xml" : modulePath + "/"
                    + module + "/sql.xml");
            //语法检测
            GrammarErrorNums = GrammarErrorNums + checkLogicGrammar(module, logicUrl);
            GrammarErrorSQLNums = GrammarErrorSQLNums + checkSqlResource(module, sqlUrl);
        }
        boolean throwError = false;
        String msg = "检查结果：";
        if (GrammarErrorNums > 0) {
            throwError = true;
            msg = msg + "\n * >>> Logic强制检查未通过，共有" + GrammarErrorNums + "项错误";
        }
        if (GrammarErrorSQLNums > 0) {
            throwError = true;
            msg = msg + "\n * >>> SQL强制检查未通过，共有" + GrammarErrorSQLNums + "项错误";
        }
        msg = msg + "\n * >>> 至理名言：" + sentence[CommonTools.getRandomNumber(0, sentence.length - 1)];
        if (throwError) {
            String errorWelcome = """
                    /***
                     * _ooOoo_
                     * o8888888o
                     * 88" . "88
                     * (| -_- |)
                     *  O\\ = /O
                     * ___/`---'\\____
                     * .   ' \\\\| |// `.
                     * / \\\\||| : |||// \\
                     * / _||||| -:- |||||- \\
                     * | | \\\\\\ - /// | |
                     * | \\_| ''\\---/'' | |
                     * \\ .-\\__ `-` ___/-. /
                     * ___`. .' /--.--\\ `. . __
                     * ."" '< `.___\\_<|>_/___.' >'"".
                     * | | : `- \\`.;`\\ _ /`;.`/ - ` : | |
                     * \\ \\ `-. \\_ __\\ /__ _/ .-` / /
                     * ======`-.____`-.___\\_____/___.-`____.-'======
                     * `=---='
                     * .............................................
                     * 佛曰：bug泛滥，我已瘫痪！
                    """;
            errorWelcome = errorWelcome + "\n * " + msg + "\n";
            LOGGER.error(errorWelcome);
            for (int i = 0; i < ERROR_ARRAY.size(); i++) {
                StringBuilder item = new StringBuilder();
                LOGGER.error(item.append(" * ").append(i).append(". ").append(ERROR_ARRAY.get(i)).toString());
            }
            LOGGER.error("==================资源强制检查完成==================\n");
            try {
                //此处睡眠是为了避免异步日志未输出完就抛出异常
                Thread.sleep(1000);
            } catch (InterruptedException e) {
                throw new RuntimeException(e);
            }
            throw new RuntimeException("Logic或SQL强制检查未通过，程序已强制中止，请在应用日志中查看错误详情");
        } else {
            String successWelcome = """
                    /***
                     *                    _ooOoo_
                     *                   o8888888o
                     *                   88" . "88
                     *                   (| -_- |)
                     *                    O\\ = /O
                     *                ____/`---'\\____
                     *              .   ' \\\\| |// `.
                     *               / \\\\||| : |||// \\
                     *             / _||||| -:- |||||- \\
                     *               | | \\\\\\ - /// | |
                     *             | \\_| ''\\---/'' | |
                     *              \\ .-\\__ `-` ___/-. /
                     *           ___`. .' /--.--\\ `. . __
                     *        ."" '< `.___\\_<|>_/___.' >'"".
                     *       | | : `- \\`.;`\\ _ /`;.`/ - ` : | |
                     *         \\ \\ `-. \\_ __\\ /__ _/ .-` / /
                     * ======`-.____`-.___\\_____/___.-`____.-'======
                     *                    `=---='
                     *
                     * .............................................
                     *          佛祖保佑             永无BUG
                     */""";
            LOGGER.info(successWelcome);
        }
    }

    /**
     * Sql资源映射检测
     */
    private int checkSqlResource(String module, String sqlUrl) {
        AtomicInteger errorNums = new AtomicInteger();
        IOTools.getStream(sqlUrl, stream -> {
            Element root = loadDocument(stream);
            List<Element> sqls = root.elements("sql");
            LOGGER.debug("检查【" + module + "】模块下Sql资源映射，共 " + sqls.size() + " 个...");
            AtomicInteger i = new AtomicInteger(1);
            AtomicInteger success = new AtomicInteger(0);
            AtomicInteger error = new AtomicInteger(0);
            sqls.forEach((it) -> {
                String index = "(" + i + "/" + sqls.size() + ")";
                String alias = it.attribute("alias").getValue();
                // 获取源程序内容
                String path = sqlMapper.getValue(alias);
                if (path == null) {
                    ERROR_ARRAY.add("Sql【" + alias + "】未配置" + index);
                    error.set(error.get() + 1);
                } else {
                    try {
                        resourceMapper.getString(ResourceType.SQL, alias, path);
                        success.set(success.get() + 1);
                    } catch (FileNotFoundException e) {
                        ERROR_ARRAY.add("Sql【" + alias + "】未找到文件：" + path);
                        error.set(error.get() + 1);
                    }
                }
                i.set(i.get() + 1);
            });
            LOGGER.debug("【" + module + "】模块语法检查完成，共 " + sqls.size() + "个,正确" + success.get() + " 个,错误 " + error.get() + " 个");
            errorNums.set(errorNums.get() + error.get());
        });
        return errorNums.get();
    }

    private Element loadDocument(InputStream stream) {
        SAXReader reader = new SAXReader();
        Document document = null;
        try {
            document = reader.read(stream);
        } catch (DocumentException e) {
            e.printStackTrace();
        }
        assert document != null;
        return document.getRootElement();
    }

    /**
     * Logic语法检测
     */
    private int checkLogicGrammar(String module, String logicUrl) {
        AtomicInteger errorNums = new AtomicInteger();
        IOTools.getStream(logicUrl, stream -> {
            Element root = loadDocument(stream);
            List<Element> logics = root.elements("logic");
            LOGGER.debug("检查【" + module + "】模块下Logic语法，共 " + logics.size() + " 个...");
            AtomicInteger i = new AtomicInteger(1);
            AtomicInteger success = new AtomicInteger(0);
            AtomicInteger error = new AtomicInteger(0);
            ResourceMapper.CACHE_MODE metaMode = ResourceMapper.mode;
            ResourceMapper.mode = ResourceMapper.CACHE_MODE.DEBUG;
            logics.forEach((it) -> {
                String index = "(" + i + "/" + logics.size() + ")";
                String alias = it.attribute("alias").getValue();
                // 获取源程序内容
                String path = logicMapper.getValue(alias);
                if (path == null) {
                    ERROR_ARRAY.add("Logic【" + alias + "】未配置" + index);
                    error.set(error.get() + 1);
                } else {
                    String source;
                    try {
                        source = resourceMapper.getString(ResourceType.LOGIC, alias, path);
                        Program pros = new Program(source);
                        // 解析
                        try {
                            pros.parse();
                            success.set(success.get() + 1);
                        } catch (RuntimeException e) {
                            String message = e.getMessage();
                            ERROR_ARRAY.add("Logic【" + alias + "】编译错误：" + message);
                            error.set(error.get() + 1);
                        }
                    } catch (FileNotFoundException e) {
                        ERROR_ARRAY.add("Logic【" + alias + "】未找到文件：" + path);
                        error.set(error.get() + 1);
                    }
                }
                i.set(i.get() + 1);
            });
            ResourceMapper.mode = metaMode;
            LOGGER.debug("【" + module + "】模块语法检查完成，共 " + logics.size() + " 个,正确 " + success.get() + " 个,错误 " + error.get() + " 个");
            errorNums.set(errorNums.get() + error.get());
        });
        return errorNums.get();
    }
}

==> af-restful-1.0.11.9-20230331.043604-2-sources.jar: com/af/v4/system/restful/utils/ResCheckUtil.java <==
package com.af.v4.system.restful.utils;

import com.af.v4.system.common.expression.core.Program;
import com.af.v4.system.common.resource.enums.ResourceType;
import com.af.v4.system.common.resource.mapper.*;
import com.af.v4.system.plugins.core.CommonTools;
import com.af.v4.system.plugins.io.IOTools;
import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.Element;
import org.dom4j.io.SAXReader;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.atomic.AtomicInteger;

/**
 * 业务资源检测工具
 *
 * @author Mrriver
 */
@Component
public class ResCheckUtil {

    private static final Logger LOGGER = LoggerFactory.getLogger(ResCheckUtil.class);

    private static final List<String> ERROR_ARRAY = new ArrayList<>();

    private static final String[] sentence = {
            "软件开发往往是这样：最开始的90%代码占用了开始的90%的开发时间；剩下10%代码同样需要90%的开发时间。",
            "这不是一个bug，这只是一个未列出来的特性。",
            "作为一个程序员，郁闷的事情是，⾯对一个代码块，却不敢去修改。更糟糕的是，这个代码块还是⾃⼰写的。",
            "在系统测试阶段查找并修复 Bug，花费的时间和工作量是让开发者自己找 Bug 的三倍。在正式上线后查找并修复 Bug，花费的时间和工作量是系统测试阶段的十倍。因此一定要让开发者自己做单元测试。",
            "不要站着调试。站着会让你的耐心减半，而且你需要集中所有精力。",
            "测试可以发现bug的存在，但不能证明bug不存在。",
            "新系统的每个新用户都会发现一类新的Bug。",
            "修复损坏的程序的第一步就是让它不断失败。",
            "有两种方法能写出没有错误的程序；但只有第三种好用。",
            "代码写的越急，程序跑得越慢。",
            "在所有的程序错误中，80%是语法错误，剩下20%里，80%是简单的逻辑错误，再剩下4%里，80%是指针错误，只有余下的0.8%才是困难的问题。"
    };

    private final ResourceMapper resourceMapper;

    private final SqlMapper sqlMapper;

    private final ModuleMapper moduleMapper;

    private final LogicMapper logicMapper;

    public ResCheckUtil(ResourceMapper resourceMapper, SqlMapper sqlMapper, ModuleMapper moduleMapper, LogicMapper logicMapper) {
        this.resourceMapper = resourceMapper;
        this.sqlMapper = sqlMapper;
        this.moduleMapper = moduleMapper;
        this.logicMapper = logicMapper;
    }

    @PostConstruct
    public void checkAll() {
        LOGGER.info("==================开始资源强制检查==================");
        // 获取所有引入的表具模块名
        Map<String, Map<String, String>> map = moduleMapper.getMap();
        int GrammarErrorNums = 0;
        int GrammarErrorSQLNums = 0;
        for (String module : map.keySet()) {
            String modulePath = map.get(module).get("path");
            String logicUrl = (modulePath == null ? module + "/logic.xml" : modulePath + "/"
                    + module + "/logic.xml");
            String sqlUrl = (modulePath == null ? module + "/sql.xml" : modulePath + "/"
                    + module + "/sql.xml");
            //语法检测
            GrammarErrorNums = GrammarErrorNums + checkLogicGrammar(module, logicUrl);
            GrammarErrorSQLNums = GrammarErrorSQLNums + checkSqlResource(module, sqlUrl);
        }
        boolean throwError = false;
        String msg = "检查结果：";
        if (GrammarErrorNums > 0) {
            throwError = true;
            msg = msg + "\n * >>> Logic强制检查未通过，共有" + GrammarErrorNums + "项错误";
        }
        if (GrammarErrorSQLNums > 0) {
            throwError = true;
            msg = msg + "\n * >>> SQL强制检查未通过，共有" + GrammarErrorSQLNums + "项错误";
        }
        msg = msg + "\n * >>> 至理名言：" + sentence[CommonTools.getRandomNumber(0, sentence.length - 1)];
        if (throwError) {
            String errorWelcome = """
                    /***
                     * _ooOoo_
                     * o8888888o
                     * 88" . "88
                     * (| -_- |)
                     *  O\\ = /O
                     * ___/`---'\\____
                     * .   ' \\\\| |// `.
                     * / \\\\||| : |||// \\
                     * / _||||| -:- |||||- \\
                     * | | \\\\\\ - /// | |
                     * | \\_| ''\\---/'' | |
                     * \\ .-\\__ `-` ___/-. /
                     * ___`. .' /--.--\\ `. . __
                     * ."" '< `.___\\_<|>_/___.' >'"".
                     * | | : `- \\`.;`\\ _ /`;.`/ - ` : | |
                     * \\ \\ `-. \\_ __\\ /__ _/ .-` / /
                     * ======`-.____`-.___\\_____/___.-`____.-'======
                     * `=---='
                     * .............................................
                     * 佛曰：bug泛滥，我已瘫痪！
                    """;
            errorWelcome = errorWelcome + "\n * " + msg + "\n";
            LOGGER.error(errorWelcome);
            for (int i = 0; i < ERROR_ARRAY.size(); i++) {
                StringBuilder item = new StringBuilder();
                LOGGER.error(item.append(" * ").append(i).append(". ").append(ERROR_ARRAY.get(i)).toString());
            }
            LOGGER.error("==================资源强制检查完成==================\n");
            try {
                //此处睡眠是为了避免异步日志未输出完就抛出异常
                Thread.sleep(1000);
            } catch (InterruptedException e) {
                throw new RuntimeException(e);
            }
            throw new RuntimeException("Logic或SQL强制检查未通过，程序已强制中止，请在应用日志中查看错误详情");
        } else {
            String successWelcome = """
                    /***
                     *                    _ooOoo_
                     *                   o8888888o
                     *                   88" . "88
                     *                   (| -_- |)
                     *                    O\\ = /O
                     *                ____/`---'\\____
                     *              .   ' \\\\| |// `.
                     *               / \\\\||| : |||// \\
                     *             / _||||| -:- |||||- \\
                     *               | | \\\\\\ - /// | |
                     *             | \\_| ''\\---/'' | |
                     *              \\ .-\\__ `-` ___/-. /
                     *           ___`. .' /--.--\\ `. . __
                     *        ."" '< `.___\\_<|>_/___.' >'"".
                     *       | | : `- \\`.;`\\ _ /`;.`/ - ` : | |
                     *         \\ \\ `-. \\_ __\\ /__ _/ .-` / /
                     * ======`-.____`-.___\\_____/___.-`____.-'======
                     *                    `=---='
                     *
                     * .............................................
                     *          佛祖保佑             永无BUG
                     */""";
            LOGGER.info(successWelcome);
        }
    }

    /**
     * Sql资源映射检测
     */
    private int checkSqlResource(String module, String sqlUrl) {
        AtomicInteger errorNums = new AtomicInteger();
        IOTools.getStream(sqlUrl, stream -> {
            Element root = loadDocument(stream);
            List<Element> sqls = root.elements("sql");
            LOGGER.info("检查【" + module + "】模块下Sql资源映射，共 " + sqls.size() + " 个...");
            AtomicInteger i = new AtomicInteger(1);
            AtomicInteger success = new AtomicInteger(0);
            AtomicInteger error = new AtomicInteger(0);
            sqls.forEach((it) -> {
                String index = "(" + i + "/" + sqls.size() + ")";
                String alias = it.attribute("alias").getValue();
                // 获取源程序内容
                AbstractResourceMapper.CommonResource<String> commonResource = sqlMapper.getValue(alias);
                String path = commonResource.getPath();
                if (path == null) {
                    ERROR_ARRAY.add("Sql【" + alias + "】未配置" + index);
                    error.set(error.get() + 1);
                } else {
                    try {
                        resourceMapper.getString(ResourceType.SQL, alias, path);
                        success.set(success.get() + 1);
                    } catch (RuntimeException e) {
                        ERROR_ARRAY.add("Sql【" + alias + "】未找到文件：" + path);
                        error.set(error.get() + 1);
                    }
                }
                i.set(i.get() + 1);
            });
            LOGGER.info("【" + module + "】模块语法检查完成，共 " + sqls.size() + "个,正确" + success.get() + " 个,错误 " + error.get() + " 个");
            errorNums.set(errorNums.get() + error.get());
        });
        return errorNums.get();
    }

    private Element loadDocument(InputStream stream) {
        SAXReader reader = new SAXReader();
        Document document = null;
        try {
            document = reader.read(stream);
        } catch (DocumentException e) {
            e.printStackTrace();
        }
        assert document != null;
        return document.getRootElement();
    }

    /**
     * Logic语法检测
     */
    private int checkLogicGrammar(String module, String logicUrl) {
        AtomicInteger errorNums = new AtomicInteger();
        IOTools.getStream(logicUrl, stream -> {
            Element root = loadDocument(stream);
            List<Element> logics = root.elements("logic");
            LOGGER.info("检查【" + module + "】模块下Logic语法，共 " + logics.size() + " 个...");
            AtomicInteger i = new AtomicInteger(1);
            AtomicInteger success = new AtomicInteger(0);
            AtomicInteger error = new AtomicInteger(0);
            ResourceMapper.CACHE_MODE metaMode = ResourceMapper.mode;
            ResourceMapper.mode = ResourceMapper.CACHE_MODE.DEBUG;
            logics.forEach((it) -> {
                String index = "(" + i + "/" + logics.size() + ")";
                String key = it.attribute("alias").getValue();
                // 获取源程序内容
                AbstractResourceMapper.CommonResource<String> commonResource = logicMapper.getValue(key);
                String alias = commonResource.getAlias();
                String path = commonResource.getPath();
                if (path == null) {
                    ERROR_ARRAY.add("Logic【" + alias + "】未配置" + index);
                    error.set(error.get() + 1);
                } else {
                    String source;
                    try {
                        source = resourceMapper.getString(ResourceType.LOGIC, alias, path);
                        Program pros = new Program(source);
                        // 解析
                        try {
                            pros.parse();
                            success.set(success.get() + 1);
                        } catch (RuntimeException e) {
                            String message = e.getMessage();
                            ERROR_ARRAY.add("Logic【" + alias + "】编译错误：" + message);
                            error.set(error.get() + 1);
                        }
                    } catch (RuntimeException e) {
                        ERROR_ARRAY.add("Logic【" + alias + "】未找到文件：" + path);
                        error.set(error.get() + 1);
                    }
                }
                i.set(i.get() + 1);
            });
            ResourceMapper.mode = metaMode;
            LOGGER.info("【" + module + "】模块语法检查完成，共 " + logics.size() + " 个,正确 " + success.get() + " 个,错误 " + error.get() + " 个");
            errorNums.set(errorNums.get() + error.get());
        });
        return errorNums.get();
    }
}
